@@ -1,3 +1,6 @@
+import six
+
+
 class LXDAPIException(Exception):
     """A generic exception for representing unexpected LXD API responses.
 
@@ -30,5 +33,15 @@ class NotFound(LXDAPIException):
     """An exception raised when an object is not found."""
 
 
+class LXDAPIExtensionNotAvailable(Exception):
+    """An exception raised when requested LXD API Extension is not present
+    on current host."""
+
+
 class ClientConnectionFailed(Exception):
     """An exception raised when the Client connection fails."""
+
+
+if six.PY2:
+    class NotADirectoryError(Exception):
+        """ An exception raised when not a directory for python2 """
